@@ -15,7 +15,7 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
-/* Copyright (C) 2005,2006 Yaacov Zamir, Nir Soffer */
+/* Copyright (C) 2005,2006,2010 Yaacov Zamir, Nir Soffer */
 
 /* 	FriBidi python binding:
     
@@ -30,18 +30,21 @@
 
 #include "pyfribidi.h"
 
+#define MAX_STR_LEN 65000
+
 static PyObject *
 _pyfribidi_log2vis (PyObject * self, PyObject * args, PyObject * kw)
 {
 	PyObject *logical = NULL;	/* input unicode or string object */
-	FriBidiCharType base = FRIBIDI_TYPE_RTL;	/* optional direction */
+	FriBidiParType base = FRIBIDI_TYPE_RTL;	/* optional direction */
 	const char *encoding = "utf-8";	/* optional input string encoding */
+	int clean = 0; /* optional flag to clean the string */
 
 	static char *kwargs[] =
-		{ "logical", "base_direction", "encoding", NULL };
+		{ "logical", "base_direction", "encoding", "clean", NULL };
 
-	if (!PyArg_ParseTupleAndKeywords (args, kw, "O|ls", kwargs,
-					  &logical, &base, &encoding))
+	if (!PyArg_ParseTupleAndKeywords (args, kw, "O|lsI", kwargs,
+					  &logical, &base, &encoding, &clean))
 		return NULL;
 
 	/* Validate base */
@@ -55,9 +58,9 @@ _pyfribidi_log2vis (PyObject * self, PyObject * args, PyObject * kw)
 	/* Check object type and delegate to one of the log2vis functions */
 
 	if (PyUnicode_Check (logical))
-		return log2vis_unicode (logical, base);
+		return log2vis_unicode (logical, base, clean);
 	else if (PyString_Check (logical))
-		return log2vis_encoded_string (logical, encoding, base);
+		return log2vis_encoded_string (logical, encoding, base, clean);
 	else
 		return PyErr_Format (PyExc_TypeError,
 				     "expected unicode or str, not %s",
@@ -82,7 +85,7 @@ _pyfribidi_log2vis (PyObject * self, PyObject * args, PyObject * kw)
 */
 
 static PyObject *
-log2vis_unicode (PyObject * unicode, FriBidiCharType base_direction)
+log2vis_unicode (PyObject * unicode, FriBidiParType base_direction, int clean)
 {
 	PyObject *logical = NULL;	/* input string encoded in utf-8 */
 	PyObject *visual = NULL;	/* output string encoded in utf-8 */
@@ -94,7 +97,7 @@ log2vis_unicode (PyObject * unicode, FriBidiCharType base_direction)
 	if (logical == NULL)
 		goto cleanup;
 
-	visual = log2vis_utf8 (logical, length, base_direction);
+	visual = log2vis_utf8 (logical, length, base_direction, clean);
 	if (visual == NULL)
 		goto cleanup;
 
@@ -125,7 +128,7 @@ log2vis_unicode (PyObject * unicode, FriBidiCharType base_direction)
 
 static PyObject *
 log2vis_encoded_string (PyObject * string, const char *encoding,
-			FriBidiCharType base_direction)
+			FriBidiParType base_direction, int clean)
 {
 	PyObject *logical = NULL;	/* logical unicode object */
 	PyObject *result = NULL;	/* output string object */
@@ -141,12 +144,12 @@ log2vis_encoded_string (PyObject * string, const char *encoding,
 		/* Shortcut for utf8 strings (little faster) */
 		result = log2vis_utf8 (string,
 				       PyUnicode_GET_SIZE (logical),
-				       base_direction);
+				       base_direction, clean);
 	else
 	{
 		/* Invoke log2vis_unicode and encode back to encoding */
 
-		PyObject *visual = log2vis_unicode (logical, base_direction);
+		PyObject *visual = log2vis_unicode (logical, base_direction, clean);
 
 		if (visual)
 		{
@@ -182,11 +185,13 @@ log2vis_encoded_string (PyObject * string, const char *encoding,
 
 static PyObject *
 log2vis_utf8 (PyObject * string, int unicode_length,
-	      FriBidiCharType base_direction)
+	      FriBidiParType base_direction, int clean)
 {
-	FriBidiChar *logical = NULL;	/* input fribidi unicode buffer */
-	FriBidiChar *visual = NULL;	/* output fribidi unicode buffer */
-	PyObject *result = NULL;	/* failure */
+	FriBidiChar *logical = NULL; /* input fribidi unicode buffer */
+	FriBidiChar *visual = NULL;	 /* output fribidi unicode buffer */
+	char *visual_utf8 = NULL;    /* output fribidi UTF-8 buffer */
+	FriBidiStrIndex new_len = 0; /* length of the UTF-8 buffer */
+	PyObject *result = NULL;	 /* failure */
 
 	/* Allocate fribidi unicode buffers */
 
@@ -219,20 +224,34 @@ log2vis_utf8 (PyObject * string, int unicode_length,
 		goto cleanup;
 	}
 
-	/* Create result string and encode the reordered string into it */
+	/* Cleanup the string if requested */
+	if (clean)
+		fribidi_remove_bidi_marks (visual, unicode_length, NULL, NULL, NULL);
 
-	result = PyString_FromStringAndSize (NULL, PyString_GET_SIZE (string));
+	/* Allocate fribidi UTF-8 buffer */
+
+	visual_utf8 = PyMem_New(char, MAX_STR_LEN);
+	if (visual_utf8 == NULL)
+	{
+		PyErr_SetString (PyExc_MemoryError,
+				"failed to allocate UTF-8 buffer");
+		goto cleanup;
+	}
+
+	/* Encode the reordered string  and create result string */
+
+	new_len = fribidi_unicode_to_utf8 (visual, unicode_length, visual_utf8);
+
+	result = PyString_FromStringAndSize (visual_utf8, new_len);
 	if (result == NULL)
 		/* XXX does it raise any error? */
 		goto cleanup;
 
-	fribidi_unicode_to_utf8 (visual, unicode_length,
-				 PyString_AS_STRING (result));
-
       cleanup:
 	/* Delete unicode buffers */
 	PyMem_Del (logical);
 	PyMem_Del (visual);
+	PyMem_Del (visual_utf8);
 
 	return result;
 }
