@@ -48,32 +48,6 @@ class UnicodeTests(unittest.TestCase):
         """ unicode: empty string """
         self.assertEqual(pyfribidi.log2vis(u''), u'')
 
-    def testBigString(self):
-        """ unicode: big string
-
-        It does not make sense to order such big strings, this just
-        checks that there are no size limits in pyfribidi.
-        """
-        # About 2MB string for default python build (ucs2)
-        big = (u'א' * 1024) * 1024
-        self.assertEqual(pyfribidi.log2vis(big), big)
-
-    def disabled_testMemoryFailure(self):
-        """ unicode: raise MemoryError when memory allocation fails
-
-        Disabled because it may not work on your machine. If you have
-        very little memory, it may fail before calling pyfribidi, if you
-        have too much memory, it may fail because no MemoryError was
-        raised.
-        """
-        # About 512MB string for default python build (ucs2)
-        try:
-            huge = ((u'x' * 1024) * 1024) * 256
-        except MemoryError:
-            raise TestSkipped('not enough memory to create test string')
-        
-        self.assertRaises(MemoryError, pyfribidi.log2vis, huge)
-
     def testDefaultDirection(self):
         """ unicode: use RTL default """
         self.assertEqual(pyfribidi.log2vis(u"hello - שלום"),
@@ -107,16 +81,6 @@ class UTF8Tests(unittest.TestCase):
         """ utf8: empty string """
         self.assertEqual(pyfribidi.log2vis(''), '')
 
-    def testBigString(self):
-        """ utf8: big string
-
-        It does not make sense to order such big strings, this just
-        checks that there are no size limits in pyfribidi.
-        """
-        # About 2MB string
-        big = ('א' * 1024) * 1024
-        self.assertEqual(pyfribidi.log2vis(big), big)
-
     def testDefaultDirection(self):
         """ utf8: use RTL default """
         self.assertEqual(pyfribidi.log2vis("hello - שלום"),
